"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FeatureFlagActionProvider = void 0;
const tslib_1 = require("tslib");
// Copyright (c) 2023 Sourcefuse Technologies
//
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT
const core_1 = require("@loopback/core");
const rest_1 = require("@loopback/rest");
const loopback4_authentication_1 = require("loopback4-authentication");
const classes_1 = require("../classes");
const keys_1 = require("../keys");
const services_1 = require("../services");
let FeatureFlagActionProvider = class FeatureFlagActionProvider {
    constructor(getMetadata, user, handlerService) {
        this.getMetadata = getMetadata;
        this.user = user;
        this.handlerService = handlerService;
        /* The `filterStrategyMapping` is an array that maps the `FilterType` to the corresponding
        `FilterStrategy`. It is used to determine the appropriate filter strategy based on the
        `FilterType` specified in the feature flag metadata provided by user through decorator. */
        this.filterStrategyMapping = [
            {
                filterType: 'AND',
                strategy: new classes_1.AndFilterStrategy(),
            },
            {
                filterType: 'OR',
                strategy: new classes_1.OrFilterStrategy(),
            },
        ];
    }
    value() {
        return () => this.action();
    }
    async action() {
        var _a, _b, _c, _d, _e;
        const metadata = await this.getMetadata();
        if (metadata && this.user) {
            const featureToCheck = [].concat(metadata.featureKey);
            if (featureToCheck.length && featureToCheck[0] === '*') {
                // skip the check and always return true
                return true;
            }
            const disabledFeatures = (_a = this.user) === null || _a === void 0 ? void 0 : _a.disabledFeatures;
            if (!disabledFeatures) {
                throw new rest_1.HttpErrors.InternalServerError('List of disabled features not passed to user');
            }
            /* The code is creating a new instance of the `FilterContext` class and setting its strategy
            based on the `operator` specified in the `metadata.options` object. */
            const context = new classes_1.FilterContext();
            context.setStrategy(this.getStrategy((_b = metadata.options) === null || _b === void 0 ? void 0 : _b.operator));
            /* Checking if the feature specified in the decorator is allowed or not based on the
            `disabledFeatures` array provided in the token. */
            const isAllowed = context.filter(featureToCheck, disabledFeatures).length <= 0;
            if (!isAllowed) {
                return false;
            }
            else {
                // check for handler
                const handler = (_c = metadata.options) === null || _c === void 0 ? void 0 : _c.handler;
                if (handler) {
                    return (_e = (_d = this.handlerService) === null || _d === void 0 ? void 0 : _d.handle(metadata, this.user)) !== null && _e !== void 0 ? _e : true;
                }
                return true;
            }
        }
        else {
            // not mandatory to apply feature check over all API
            return true;
        }
    }
    getStrategy(operator) {
        var _a, _b;
        /* Filtering strategy based on the `operator` specified in the decorator. It uses the
        `filterStrategyMapping` array to map the `FilterType` to the corresponding `FilterStrategy`.
         If strategy is not provided in the decorator default OR strategy will work */
        return ((_b = (_a = this.filterStrategyMapping.find(value => value.filterType === operator)) === null || _a === void 0 ? void 0 : _a.strategy) !== null && _b !== void 0 ? _b : new classes_1.OrFilterStrategy());
    }
};
exports.FeatureFlagActionProvider = FeatureFlagActionProvider;
exports.FeatureFlagActionProvider = FeatureFlagActionProvider = tslib_1.__decorate([
    tslib_1.__param(0, core_1.inject.getter(keys_1.StrategyBindings.METADATA)),
    tslib_1.__param(1, (0, core_1.inject)(loopback4_authentication_1.AuthenticationBindings.CURRENT_USER)),
    tslib_1.__param(2, (0, core_1.service)(services_1.FeatureHandlerService)),
    tslib_1.__metadata("design:paramtypes", [Function, Object, services_1.FeatureHandlerService])
], FeatureFlagActionProvider);
//# sourceMappingURL=feature-flag-action.provider.js.map